import bpy
from bpy.props import IntProperty, StringProperty, FloatProperty, FloatVectorProperty, BoolProperty, EnumProperty

from .. utils import (
    addon,
    icons,
    theme_utils,
)

preview_collections = {}


class ARMORED_PT_theme_candy_preferences(bpy.types.AddonPreferences):
	bl_idname = addon.name()

	# UPDATE FUNCTIONS >>
	def update_mode(self, context):
		self.update_neutral_sliders(context)

	def update_neutral_sliders(self, context):
		ui = context.preferences.themes[0].user_interface

		if self.neutral_sliders:
			ui.wcol_numslider.item = (.2, .2, .2, 1)
			ui.wcol_numslider.text_sel = (1, 1, 1)
		else:
			ui.wcol_numslider.item 	   = ui.wcol_regular.inner_sel
			ui.wcol_numslider.text_sel = ui.wcol_regular.text_sel


	# ADDON PREFERENCES:
	# ===============================
	mode : EnumProperty(
		name='Customization Level', 
		default='BASIC',
		update=update_mode,
		items=[ 
			('BASIC',    'Basic',    'Use the simple menu with only the important settings (recommended).'),
			('ADVANCED', 'Advanced', 'Use the advanced menu with extra settings.'),   
		]
	)
	
	neutral_sliders : BoolProperty(
		name='Neutral UI Sliders', 
		default=False,
		description='\u2022 Enabled: Force UI Sliders to use a dark gray color for readability. \n\u2022 Disabled: UI Sliders are colorized normaly',
		update=update_neutral_sliders)
	
	# sidebar_name : StringProperty(name='Theme Candy', default='Theme Candy', description='Name of the Theme Candy Sidebar Panel.')
	
	# collapse_panels : BoolProperty(name='Collapse Panels', default=False, description='Force the individual Theme Candy Panels to be closed on startup.')
	

	# INTERNAL PREFERENCES:
	# ===============================
	lock_updates   : BoolProperty(default=False)


	def draw(self, context):
		layout = self.layout
		layout.use_property_split = True
		
		web_icons = preview_collections['web_icons']


		col = layout.column(align=True)
		row = col.row()
		row.scale_y = 1.5
		row.operator('wm.url_open', text='Quickstart Video', icon='FILE_MOVIE').url = 'https://www.youtube.com/watch?v=3xpxrWC6Z0k&ab_channel=ArmoredColony'
		# col.separator()

		row = col.row(align=True)
		row.scale_y = 1.5
		row.operator('wm.url_open', text='Armored Colony',	icon_value=web_icons['armored_colony32'].icon_id).url 	= 'https://armoredColony.com'
		row.operator('wm.url_open', text='Artstation', 		icon_value=web_icons['artstation32'].icon_id).url 	= 'https://armoredColony.artstation.com'
		row.operator('wm.url_open', text='Blender Market', 	icon_value=web_icons['blender_market32'].icon_id).url 	= 'https://blendermarket.com/creators/armoredcolony'
		row.operator('wm.url_open', text='GitHub', 		icon_value=web_icons['github32'].icon_id).url 		= 'https://github.com/ArmoredColony/ARMORED-Toolkit'
		row.operator('wm.url_open', text='Youtube', 		icon_value=web_icons['youtube32'].icon_id).url 		= 'https://www.youtube.com/armoredColony'
		layout.separator()

		box = layout.box()
		col = box.column(align=True)
		row = col.row()
		row.prop(self, 'mode', expand=True)
		
		col.separator()

		col.prop(self, 'neutral_sliders')

		col.separator()


class ThemeCandyProperties(bpy.types.PropertyGroup):
	execute = theme_utils.SharedExecute.execute

	# USER INTERFACE:
	# ===============================

	ui_selected	: FloatVectorProperty(update=execute, name='UI Selected',	subtype='COLOR_GAMMA', min=0, max=1, size=4)
	ui_unselected	: FloatVectorProperty(update=execute, name='UI',	subtype='COLOR_GAMMA', min=0, max=1, size=4)
	text_selected	: FloatVectorProperty(update=execute, name='Text Selected',	subtype='COLOR_GAMMA', min=0, max=1, size=3)    # 3 is default but explicit is always good idea.
	text_unselected	: FloatVectorProperty(update=execute, name='Text',	subtype='COLOR_GAMMA', min=0, max=1, size=3)    # 3 is default but explicit is always good idea.
	modifier_active	: FloatVectorProperty(update=execute, name='Active Modifier',	subtype='COLOR_GAMMA', min=0, max=1, size=4)

	roundness	: FloatProperty(update=execute, name='UI Roundness', min=0, max=1)
	
	# VIEWPORT SELECTIONS:
	# ===============================

	object_active    : FloatVectorProperty(update=execute, name='Active Object',	subtype='COLOR_GAMMA', min=0, max=1, size=3)
	object_select    : FloatVectorProperty(update=execute, name='Selected Objects',	subtype='COLOR_GAMMA', min=0, max=1, size=3)

	vertex           : FloatVectorProperty(update=execute, name='Vertex',           subtype='COLOR_GAMMA', min=0, max=1, size=3)
	vertex_select    : FloatVectorProperty(update=execute, name='Vertex Select',    subtype='COLOR_GAMMA', min=0, max=1, size=3)
	edge_select      : FloatVectorProperty(update=execute, name='Edge Select',      subtype='COLOR_GAMMA', min=0, max=1, size=3)
	edge_mode_select : FloatVectorProperty(update=execute, name='Edge Mode Select', subtype='COLOR_GAMMA', min=0, max=1, size=3)
	face_select      : FloatVectorProperty(update=execute, name='Face Select',      subtype='COLOR_GAMMA', min=0, max=1, size=4)
	face_mode_select : FloatVectorProperty(update=execute, name='Face Mode Select', subtype='COLOR_GAMMA', min=0, max=1, size=4)
	element_active   : FloatVectorProperty(update=execute, name='Active Element',   subtype='COLOR_GAMMA', min=0, max=1, size=4)
	global_select    : FloatVectorProperty(update=execute, name='Selection Color',  subtype='COLOR_GAMMA', min=0, max=1, size=3)
	
	# VIEWPORT BACKGROUND:
	# ===============================

	gradient_high    : FloatVectorProperty(update=execute, name='Gradient High',    subtype='COLOR_GAMMA', min=0, max=1, size=3)
	gradient_low     : FloatVectorProperty(update=execute, name='Gradient Low',     subtype='COLOR_GAMMA', min=0, max=1, size=3)
	grid_color       : FloatVectorProperty(update=execute, name='Grid Color',       subtype='COLOR_GAMMA', min=0, max=1, size=4)

	axis_x           : FloatVectorProperty(update=execute, name='Axis X',           subtype='COLOR_GAMMA', min=0, max=1, size=3)
	axis_y           : FloatVectorProperty(update=execute, name='Axis Y',           subtype='COLOR_GAMMA', min=0, max=1, size=3)
	axis_z           : FloatVectorProperty(update=execute, name='Axis Z',           subtype='COLOR_GAMMA', min=0, max=1, size=3)

	background_type  : EnumProperty(update=execute, name='Background Type', default='SINGLE_COLOR',
		items=[ 
			('SINGLE_COLOR', 'Single Color',    'Use a solid color as viewport background'),
			('LINEAR',       'Linear Gradient', 'Use a screen space vertical linear gradient as viewport background'),
			('RADIAL',       'Vignette',        'Use a radial gradient as viewport background'),    
		],
	)

	# OTHER PROPERTIES:
	# ===============================

	text_shadow_size	: IntProperty(update=execute,   name='Text Shadow Size',  default=1,	min=0, max=5,	description='Text shadow size on selected UI elements')
	text_shadow_alpha	: FloatProperty(update=execute, name='Text Shadow Alpha', default=0.5,	min=0, max=1,	description='Text shadow transparency on selected UI elements')

	vertex_size		: IntProperty(update=execute,   name='Vertex Size',       default=3,	min=1, max=32,	description='How big are vertices drawn in the viewport')
	face_alpha		: FloatProperty(update=execute, name='Face Alpha',        default=1,	min=0, max=1,	description='Transparency of face selections')
	outliner_zebra		: BoolProperty(update=execute,  name='Zebra Pattern',     default=True,			description='Display striped rows in the outliner')

	# OUTLINER:
	# ===============================

	outliner_active_highlight	: FloatVectorProperty(update=execute, name='Highlight Active',		subtype='COLOR_GAMMA', min=0, max=1, size=3)
	outliner_active_text		: FloatVectorProperty(update=execute, name='Text Active',		subtype='COLOR_GAMMA', min=0, max=1, size=3)
	outliner_selected_highlight	: FloatVectorProperty(update=execute, name='Highlight Selected',	subtype='COLOR_GAMMA', min=0, max=1, size=3)
	outliner_selected_text		: FloatVectorProperty(update=execute, name='Text Selected',		subtype='COLOR_GAMMA', min=0, max=1, size=3)


	# LINK WITH UI COLOR:
	# ===============================

	vertex_select_link		: BoolProperty(update=execute, name='Link with UI color', 		default=False)
	edge_select_link		: BoolProperty(update=execute, name='Link with UI color', 		default=False)
	edge_mode_select_link		: BoolProperty(update=execute, name='Link with UI color', 		default=False)
	face_select_link		: BoolProperty(update=execute, name='Link with UI color', 		default=False)
	face_mode_select_link		: BoolProperty(update=execute, name='Link with UI color', 		default=False)
	element_active_link		: BoolProperty(update=execute, name='Link with UI color', 		default=False)
	global_select_link		: BoolProperty(update=execute, name='Link with UI color', 		default=False)
	modifier_active_link		: BoolProperty(update=execute, name='Link with UI color', 		default=False)
	object_active_link		: BoolProperty(update=execute, name='Link with UI color', 		default=False)
	axis_x_link			: BoolProperty(update=execute, name='Link with UI color', 		default=False)

	outliner_active_highlight_link	: BoolProperty(update=execute, name='Link with UI selected color',   	default=False)
	outliner_active_text_link	: BoolProperty(update=execute, name='Link with Text selected color', 	default=False)


	# ICON COLORS:
	# ===============================

	icon_scene		: FloatVectorProperty(update=execute, name='Scene',		subtype='COLOR_GAMMA', min=0, max=1, size=4)
	icon_collection		: FloatVectorProperty(update=execute, name='Collection',	subtype='COLOR_GAMMA', min=0, max=1, size=4)
	icon_object		: FloatVectorProperty(update=execute, name='Object',		subtype='COLOR_GAMMA', min=0, max=1, size=4)
	icon_object_data	: FloatVectorProperty(update=execute, name='Object Data',	subtype='COLOR_GAMMA', min=0, max=1, size=4)
	icon_modifier		: FloatVectorProperty(update=execute, name='Modifier',		subtype='COLOR_GAMMA', min=0, max=1, size=4)
	icon_shading		: FloatVectorProperty(update=execute, name='Shading',		subtype='COLOR_GAMMA', min=0, max=1, size=4)
	icon_folder		: FloatVectorProperty(update=execute, name='File Folders',	subtype='COLOR_GAMMA', min=0, max=1, size=4)


	# AREA COLORS:
	# ===============================

	# Backgrounds mainly, but also a few Headers or any sub-elements I want to be the same as the background.

	dopesheet_editor	: FloatVectorProperty(update=execute, name='Dopesheet',		subtype='COLOR_GAMMA', min=0, max=1, size=3, description='Background Color')
	file_browser		: FloatVectorProperty(update=execute, name='File Browser',	subtype='COLOR_GAMMA', min=0, max=1, size=3, description='Background Color')
	graph_editor		: FloatVectorProperty(update=execute, name='Graph Editor',	subtype='COLOR_GAMMA', min=0, max=1, size=3, description='Background Color')
	image_editor		: FloatVectorProperty(update=execute, name='UV/Image Editor',	subtype='COLOR_GAMMA', min=0, max=1, size=3, description='Background Color')
	info			: FloatVectorProperty(update=execute, name='Info',		subtype='COLOR_GAMMA', min=0, max=1, size=3, description='Background Color')
	clip_editor		: FloatVectorProperty(update=execute, name='Clip Editor',	subtype='COLOR_GAMMA', min=0, max=1, size=3, description='Background Color')
	node_editor		: FloatVectorProperty(update=execute, name='Node Editor',	subtype='COLOR_GAMMA', min=0, max=1, size=3, description='Background Color')
	nla_editor		: FloatVectorProperty(update=execute, name='NLA Editor',	subtype='COLOR_GAMMA', min=0, max=1, size=3, description='Background Color')
	outliner		: FloatVectorProperty(update=execute, name='Outliner',		subtype='COLOR_GAMMA', min=0, max=1, size=3, description='Background Color')
	preferences		: FloatVectorProperty(update=execute, name='Preferences',	subtype='COLOR_GAMMA', min=0, max=1, size=3, description='Background Color')
	properties		: FloatVectorProperty(update=execute, name='Properties',	subtype='COLOR_GAMMA', min=0, max=1, size=3, description='Background Color')
	console			: FloatVectorProperty(update=execute, name='Console',		subtype='COLOR_GAMMA', min=0, max=1, size=3, description='Background Color')
	spreadsheet		: FloatVectorProperty(update=execute, name='Spreadsheet',	subtype='COLOR_GAMMA', min=0, max=1, size=3, description='Background Color')
	text_editor		: FloatVectorProperty(update=execute, name='Text Editor',	subtype='COLOR_GAMMA', min=0, max=1, size=3, description='Background Color')
	sequence_editor		: FloatVectorProperty(update=execute, name='Sequence Editor',	subtype='COLOR_GAMMA', min=0, max=1, size=3, description='Background Color')

	# Panels:
	
	dopesheet_editor_panel	: FloatVectorProperty(update=execute, name='Dopesheet',		subtype='COLOR_GAMMA', min=0, max=1, size=4, description='Panel Color')
	file_browser_panel	: FloatVectorProperty(update=execute, name='File Browser',	subtype='COLOR_GAMMA', min=0, max=1, size=4, description='Panel Color')
	graph_editor_panel	: FloatVectorProperty(update=execute, name='Graph Editor',	subtype='COLOR_GAMMA', min=0, max=1, size=4, description='Panel Color')
	image_editor_panel	: FloatVectorProperty(update=execute, name='UV_Image Editor',	subtype='COLOR_GAMMA', min=0, max=1, size=4, description='Panel Color')
	info_panel		: FloatVectorProperty(update=execute, name='Info',		subtype='COLOR_GAMMA', min=0, max=1, size=4, description='Panel Color')
	clip_editor_panel	: FloatVectorProperty(update=execute, name='Clip Editor',	subtype='COLOR_GAMMA', min=0, max=1, size=4, description='Panel Color')
	node_editor_panel	: FloatVectorProperty(update=execute, name='Node Editor',	subtype='COLOR_GAMMA', min=0, max=1, size=4, description='Panel Color')
	nla_editor_panel	: FloatVectorProperty(update=execute, name='NLA Editor',	subtype='COLOR_GAMMA', min=0, max=1, size=4, description='Panel Color')
	outliner_panel		: FloatVectorProperty(update=execute, name='Outliner',		subtype='COLOR_GAMMA', min=0, max=1, size=4, description='Panel Color')
	preferences_panel	: FloatVectorProperty(update=execute, name='Preferences',	subtype='COLOR_GAMMA', min=0, max=1, size=4, description='Panel Color')
	properties_panel	: FloatVectorProperty(update=execute, name='Properties',	subtype='COLOR_GAMMA', min=0, max=1, size=4, description='Panel Color')
	console_panel		: FloatVectorProperty(update=execute, name='Console',		subtype='COLOR_GAMMA', min=0, max=1, size=4, description='Panel Color')
	spreadsheet_panel	: FloatVectorProperty(update=execute, name='Spreadsheet',	subtype='COLOR_GAMMA', min=0, max=1, size=4, description='Panel Color')
	text_editor_panel	: FloatVectorProperty(update=execute, name='Text Editor',	subtype='COLOR_GAMMA', min=0, max=1, size=4, description='Panel Color')
	sequence_editor_panel	: FloatVectorProperty(update=execute, name='Sequence Editor',	subtype='COLOR_GAMMA', min=0, max=1, size=4, description='Panel Color')

	# Headers:

	console_header		: FloatVectorProperty(update=execute, name='Console',		subtype='COLOR_GAMMA', min=0, max=1, size=4, description='Header Color')
	text_editor_header	: FloatVectorProperty(update=execute, name='Text Editor',	subtype='COLOR_GAMMA', min=0, max=1, size=4, description='Header Color')

	# Other:

	node_editor_grid	: FloatVectorProperty(update=execute, name='Node Editor',	subtype='COLOR_GAMMA', min=0, max=1, size=3, description='Grid Color')
	editor_outline		: FloatVectorProperty(update=execute, name='Editor Outline',	subtype='COLOR_GAMMA', min=0, max=1, size=3, description='Background Color')


classes = (
	ARMORED_PT_theme_candy_preferences,
	ThemeCandyProperties,
)

def register():
	for cls in classes:
		bpy.utils.register_class(cls)

	icons.load_icons(preview_collections)
	bpy.types.WindowManager.themecandy = bpy.props.PointerProperty(type=ThemeCandyProperties)
	theme_utils.init_color_properties(bpy.context)


def unregister():
	for cls in classes:
		bpy.utils.unregister_class(cls)
	icons.unload_icons(preview_collections)
